==> octave-7.3.0.tar.gz: octave-7.3.0/libinterp/operators/op-bm-bm.cc <==
////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 1996-2022 The Octave Project Developers
//
// See the file COPYRIGHT.md in the top-level directory of this
// distribution or <https://octave.org/copyright/>.
//
// This file is part of Octave.
//
// Octave is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Octave is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Octave; see the file COPYING.  If not, see
// <https://www.gnu.org/licenses/>.
//
////////////////////////////////////////////////////////////////////////

#if defined (HAVE_CONFIG_H)
#  include "config.h"
#endif

#include "errwarn.h"
#include "ovl.h"
#include "ov.h"
#include "ov-bool-mat.h"
#include "ov-scalar.h"
#include "ov-range.h"
#include "ov-re-mat.h"
#include "ov-flt-re-mat.h"
#include "ov-re-sparse.h"
#include "ov-str-mat.h"
#include "ov-int8.h"
#include "ov-int16.h"
#include "ov-int32.h"
#include "ov-int64.h"
#include "ov-uint8.h"
#include "ov-uint16.h"
#include "ov-uint32.h"
#include "ov-uint64.h"
#include "ov-typeinfo.h"
#include "ov-null-mat.h"
#include "ops.h"
#include "xdiv.h"
#include "xpow.h"

OCTAVE_NAMESPACE_BEGIN

// unary bool matrix ops.

DEFNDUNOP_OP (not, bool_matrix, bool_array, !)
DEFNDUNOP_OP (uplus, bool_matrix, array, +)
DEFNDUNOP_OP (uminus, bool_matrix, array, -)

DEFNCUNOP_METHOD (invert, bool_matrix, invert)

DEFUNOP (transpose, bool_matrix)
{
  const octave_bool_matrix& v = dynamic_cast<const octave_bool_matrix&> (a);

  if (v.ndims () > 2)
    error ("transpose not defined for N-D objects");

  return octave_value (v.bool_matrix_value ().transpose ());
}

// bool matrix by bool matrix ops.

DEFNDBINOP_FN (eq, bool_matrix, bool_matrix, bool_array, bool_array, mx_el_eq)
DEFNDBINOP_FN (ne, bool_matrix, bool_matrix, bool_array, bool_array, mx_el_ne)

DEFNDBINOP_FN (el_and, bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_and)

DEFNDBINOP_FN (el_or,  bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_or)

DEFNDBINOP_FN (el_not_and, bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_not_and)

DEFNDBINOP_FN (el_not_or,  bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_not_or)

DEFNDBINOP_FN (el_and_not, bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_and_not)

DEFNDBINOP_FN (el_or_not,  bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_or_not)

DEFNDCATOP_FN (bm_bm, bool_matrix, bool_matrix, bool_array, bool_array, concat)
DEFNDCATOP_FN (bm_m, bool_matrix, matrix, array, array, concat)
DEFNDCATOP_FN (m_bm, matrix, bool_matrix, array, array, concat)
DEFNDCATOP_FN (bm_fm, bool_matrix, float_matrix, float_array, float_array,
               concat)
DEFNDCATOP_FN (fm_bm, float_matrix, bool_matrix, float_array, float_array,
               concat)

DEFNDASSIGNOP_FN (assign, bool_matrix, bool_matrix, bool_array, assign)
DEFNDASSIGNOP_FNOP (assign_and, bool_matrix, bool_matrix, bool_array,
                    mx_el_and_assign)
DEFNDASSIGNOP_FNOP (assign_or, bool_matrix, bool_matrix, bool_array,
                    mx_el_or_assign)

DEFNULLASSIGNOP_FN (null_assign, bool_matrix, delete_elements)

static octave_value
oct_assignop_conv_and_assign (octave_base_value& a1,
                              const octave_value_list& idx,
                              const octave_base_value& a2)
{
  octave_bool_matrix& v1 = dynamic_cast<octave_bool_matrix&> (a1);

  // FIXME: perhaps add a warning for this conversion
  //        if the values are not all 0 or 1?

  boolNDArray v2 = a2.bool_array_value (true);

  v1.assign (idx, v2);

  return octave_value ();
}

void
install_bm_bm_ops (octave::type_info& ti)
{
  INSTALL_UNOP_TI (ti, op_not, octave_bool_matrix, not);
  INSTALL_UNOP_TI (ti, op_uplus, octave_bool_matrix, uplus);
  INSTALL_UNOP_TI (ti, op_uminus, octave_bool_matrix, uminus);
  INSTALL_UNOP_TI (ti, op_transpose, octave_bool_matrix, transpose);
  INSTALL_UNOP_TI (ti, op_hermitian, octave_bool_matrix, transpose);

  INSTALL_NCUNOP_TI (ti, op_not, octave_bool_matrix, invert);

  INSTALL_BINOP_TI (ti, op_eq, octave_bool_matrix, octave_bool_matrix, eq);
  INSTALL_BINOP_TI (ti, op_ne, octave_bool_matrix, octave_bool_matrix, ne);

  INSTALL_BINOP_TI (ti, op_el_and, octave_bool_matrix, octave_bool_matrix, el_and);
  INSTALL_BINOP_TI (ti, op_el_or, octave_bool_matrix, octave_bool_matrix, el_or);
  INSTALL_BINOP_TI (ti, op_el_not_and, octave_bool_matrix, octave_bool_matrix,
                    el_not_and);
  INSTALL_BINOP_TI (ti, op_el_not_or, octave_bool_matrix, octave_bool_matrix,
                    el_not_or);
  INSTALL_BINOP_TI (ti, op_el_and_not, octave_bool_matrix, octave_bool_matrix,
                    el_and_not);
  INSTALL_BINOP_TI (ti, op_el_or_not, octave_bool_matrix, octave_bool_matrix,
                    el_or_not);

  INSTALL_CATOP_TI (ti, octave_bool_matrix, octave_bool_matrix, bm_bm);
  INSTALL_CATOP_TI (ti, octave_bool_matrix, octave_matrix, bm_m);
  INSTALL_CATOP_TI (ti, octave_matrix, octave_bool_matrix, m_bm);
  INSTALL_CATOP_TI (ti, octave_bool_matrix, octave_float_matrix, bm_fm);
  INSTALL_CATOP_TI (ti, octave_float_matrix, octave_bool_matrix, fm_bm);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_bool_matrix,
                       assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_char_matrix_str,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix,
                       octave_char_matrix_sq_str,
                       conv_and_assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_range,
                       conv_and_assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_sparse_matrix,
                       conv_and_assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_int8_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_int16_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_int32_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_int64_matrix,
                       conv_and_assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_uint8_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_uint16_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_uint32_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_uint64_matrix,
                       conv_and_assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_null_matrix,
                       null_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_null_str,
                       null_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_null_sq_str,
                       null_assign);

  INSTALL_ASSIGNOP_TI (ti, op_el_and_eq, octave_bool_matrix, octave_bool_matrix,
                       assign_and);
  INSTALL_ASSIGNOP_TI (ti, op_el_or_eq, octave_bool_matrix, octave_bool_matrix,
                       assign_or);
}

OCTAVE_NAMESPACE_END

==> octave-11.0.92.tar.gz: octave-11.0.92/libinterp/operators/op-bm-bm.cc <==
////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 1996-2026 The Octave Project Developers
//
// See the file COPYRIGHT.md in the top-level directory of this
// distribution or <https://octave.org/copyright/>.
//
// This file is part of Octave.
//
// Octave is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Octave is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Octave; see the file COPYING.  If not, see
// <https://www.gnu.org/licenses/>.
//
////////////////////////////////////////////////////////////////////////

#if defined (HAVE_CONFIG_H)
#  include "config.h"
#endif

#include "errwarn.h"
#include "ovl.h"
#include "ov.h"
#include "ov-bool-mat.h"
#include "ov-scalar.h"
#include "ov-range.h"
#include "ov-re-mat.h"
#include "ov-flt-re-mat.h"
#include "ov-re-sparse.h"
#include "ov-str-mat.h"
#include "ov-int8.h"
#include "ov-int16.h"
#include "ov-int32.h"
#include "ov-int64.h"
#include "ov-uint8.h"
#include "ov-uint16.h"
#include "ov-uint32.h"
#include "ov-uint64.h"
#include "ov-typeinfo.h"
#include "ov-null-mat.h"
#include "ops.h"
#include "xdiv.h"
#include "xpow.h"

OCTAVE_BEGIN_NAMESPACE(octave)

// unary bool matrix ops.

DEFNDUNOP_OP (not, bool_matrix, bool_array, !)
DEFNDUNOP_OP (uplus, bool_matrix, array, +)
DEFNDUNOP_OP (uminus, bool_matrix, array, -)

DEFNCUNOP_METHOD (invert, bool_matrix, invert)

DEFUNOP (transpose, bool_matrix)
{
  OCTAVE_CAST_BASE_VALUE (const octave_bool_matrix&, v, a);

  if (v.ndims () > 2)
    error ("transpose not defined for N-D objects");

  return octave_value (v.bool_matrix_value ().transpose ());
}

// bool matrix by bool matrix ops.

DEFNDBINOP_FN (eq, bool_matrix, bool_matrix, bool_array, bool_array, mx_el_eq)
DEFNDBINOP_FN (ne, bool_matrix, bool_matrix, bool_array, bool_array, mx_el_ne)

DEFNDBINOP_FN (el_and, bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_and)

DEFNDBINOP_FN (el_or,  bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_or)

DEFNDBINOP_FN (el_not_and, bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_not_and)

DEFNDBINOP_FN (el_not_or,  bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_not_or)

DEFNDBINOP_FN (el_and_not, bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_and_not)

DEFNDBINOP_FN (el_or_not,  bool_matrix, bool_matrix, bool_array, bool_array,
               mx_el_or_not)

DEFNDCATOP_FN (bm_bm, bool_matrix, bool_matrix, bool_array, bool_array, concat)
DEFNDCATOP_FN (bm_m, bool_matrix, matrix, array, array, concat)
DEFNDCATOP_FN (m_bm, matrix, bool_matrix, array, array, concat)
DEFNDCATOP_FN (bm_fm, bool_matrix, float_matrix, float_array, float_array,
               concat)
DEFNDCATOP_FN (fm_bm, float_matrix, bool_matrix, float_array, float_array,
               concat)

DEFNDASSIGNOP_FN (assign, bool_matrix, bool_matrix, bool_array, assign)
DEFNDASSIGNOP_FNOP (assign_and, bool_matrix, bool_matrix, bool_array,
                    mx_el_and_assign)
DEFNDASSIGNOP_FNOP (assign_or, bool_matrix, bool_matrix, bool_array,
                    mx_el_or_assign)

DEFNULLASSIGNOP_FN (null_assign, bool_matrix, delete_elements)

static octave_value
oct_assignop_conv_and_assign (octave_base_value& a1,
                              const octave_value_list& idx,
                              const octave_base_value& a2)
{
  OCTAVE_CAST_BASE_VALUE (octave_bool_matrix&, v1, a1);

  boolNDArray v2 = a2.bool_array_value ();

  v1.assign (idx, v2);

  return octave_value ();
}

void
install_bm_bm_ops (octave::type_info& ti)
{
  INSTALL_UNOP_TI (ti, op_not, octave_bool_matrix, not);
  INSTALL_UNOP_TI (ti, op_uplus, octave_bool_matrix, uplus);
  INSTALL_UNOP_TI (ti, op_uminus, octave_bool_matrix, uminus);
  INSTALL_UNOP_TI (ti, op_transpose, octave_bool_matrix, transpose);
  INSTALL_UNOP_TI (ti, op_hermitian, octave_bool_matrix, transpose);

  INSTALL_NCUNOP_TI (ti, op_not, octave_bool_matrix, invert);

  INSTALL_BINOP_TI (ti, op_eq, octave_bool_matrix, octave_bool_matrix, eq);
  INSTALL_BINOP_TI (ti, op_ne, octave_bool_matrix, octave_bool_matrix, ne);

  INSTALL_BINOP_TI (ti, op_el_and, octave_bool_matrix, octave_bool_matrix, el_and);
  INSTALL_BINOP_TI (ti, op_el_or, octave_bool_matrix, octave_bool_matrix, el_or);
  INSTALL_BINOP_TI (ti, op_el_not_and, octave_bool_matrix, octave_bool_matrix,
                    el_not_and);
  INSTALL_BINOP_TI (ti, op_el_not_or, octave_bool_matrix, octave_bool_matrix,
                    el_not_or);
  INSTALL_BINOP_TI (ti, op_el_and_not, octave_bool_matrix, octave_bool_matrix,
                    el_and_not);
  INSTALL_BINOP_TI (ti, op_el_or_not, octave_bool_matrix, octave_bool_matrix,
                    el_or_not);

  INSTALL_CATOP_TI (ti, octave_bool_matrix, octave_bool_matrix, bm_bm);
  INSTALL_CATOP_TI (ti, octave_bool_matrix, octave_matrix, bm_m);
  INSTALL_CATOP_TI (ti, octave_matrix, octave_bool_matrix, m_bm);
  INSTALL_CATOP_TI (ti, octave_bool_matrix, octave_float_matrix, bm_fm);
  INSTALL_CATOP_TI (ti, octave_float_matrix, octave_bool_matrix, fm_bm);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_bool_matrix,
                       assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_char_matrix_str,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix,
                       octave_char_matrix_sq_str,
                       conv_and_assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_range,
                       conv_and_assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_sparse_matrix,
                       conv_and_assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_int8_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_int16_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_int32_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_int64_matrix,
                       conv_and_assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_uint8_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_uint16_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_uint32_matrix,
                       conv_and_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_uint64_matrix,
                       conv_and_assign);

  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_null_matrix,
                       null_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_null_str,
                       null_assign);
  INSTALL_ASSIGNOP_TI (ti, op_asn_eq, octave_bool_matrix, octave_null_sq_str,
                       null_assign);

  INSTALL_ASSIGNOP_TI (ti, op_el_and_eq, octave_bool_matrix, octave_bool_matrix,
                       assign_and);
  INSTALL_ASSIGNOP_TI (ti, op_el_or_eq, octave_bool_matrix, octave_bool_matrix,
                       assign_or);
}

OCTAVE_END_NAMESPACE(octave)
